use actix_web::{
    error::{ErrorBadRequest, ErrorInternalServerError},
    web, Responder,
};
use argon2::{Argon2, PasswordHash, PasswordVerifier};
use sea_orm::{ColumnTrait, EntityTrait, QueryFilter};
use serde::Deserialize;

use crate::AppState;

use super::user::UserWithoutPassword;

#[derive(Deserialize)]
pub struct LoginRequest {
    email: String,
    password: String,
}

pub struct AuthController;

impl AuthController {
    pub async fn login(
        state: web::Data<AppState>,
        login: web::Json<LoginRequest>,
    ) -> actix_web::Result<impl Responder> {
        let db = &state.db;
        let login = login.into_inner();

        let user = entity::user::Entity::find()
            .filter(entity::user::Column::Email.eq(login.email))
            .one(db)
            .await
            .map_err(ErrorInternalServerError)?
            .ok_or(ErrorBadRequest("Login Failed"))?;

        let argon2 = Argon2::default();
        let parsed_hash = PasswordHash::new(&user.hash).map_err(ErrorInternalServerError)?;
        argon2
            .verify_password(login.password.as_bytes(), &parsed_hash)
            .map_err(ErrorBadRequest)?;

        Ok(web::Json(UserWithoutPassword::from(user)))
    }
}
